part of '../rm.dart';

class InjectedImp<T> extends Injected<T> {
  InjectedImp({
    required dynamic Function()? creator,
    T? initialState,
    this.onInitialized,
    this.onDisposed,
    this.onSetState,
    this.onWaiting,
    this.onDataForSideEffect,
    this.onError,
    bool isAsyncInjected = false,
    this.dependsOn,
    this.undoStackLength = 0,
    PersistState<T> Function()? persist,
    this.middleSnapState,
    bool isLazy = true,
    this.debugPrintWhenNotifiedPreMessage,
    this.toDebugString,
    bool autoDisposeWhenNotUsed = true,
  }) : _isAsyncInjected = isAsyncInjected {
    if (undoStackLength > 0 || persist != null) {
      undoRedoPersistState = UndoRedoPersistState(
        undoStackLength: undoStackLength,
        persistanceProvider: persist?.call(),
      );
    }

    if (creator != null) {
      //if creator is null, it must be defined latter before initializing
      _reactiveModelState = ReactiveModelBase<T>(
        creator: creator,
        initialState: initialState,
        initializer: initialize,
        autoDisposeWhenNotUsed: autoDisposeWhenNotUsed,
        debugPrintWhenNotifiedPreMessage: debugPrintWhenNotifiedPreMessage,
      );
      if (!isLazy) {
        initialize();
      }
    }
  }
  ReactiveModelBase<T> get reactiveModelState => _reactiveModelState;
  set reactiveModelState(ReactiveModelBase<T> rm) => _reactiveModelState = rm;
  UndoRedoPersistState<T>? undoRedoPersistState;

  final SnapState<T>? Function(MiddleSnapState<T> middleSnap)? middleSnapState;
  final On<void>? onSetState;
  final String? debugPrintWhenNotifiedPreMessage;
  final Object? Function(T?)? toDebugString;

  void Function(T? s)? onInitialized;
  final void Function(T s)? onDisposed;
  final void Function()? onWaiting;
  final void Function(T s)? onDataForSideEffect;
  // final On<void>? onSetState;
  final void Function(dynamic e, StackTrace? s)? onError;

  @override
  bool get isIdle {
    OnReactiveState.addToObs?.call(this);
    return _reactiveModelState.snapState.isIdle;
  }

  @override
  bool get isWaiting {
    OnReactiveState.addToObs?.call(this);
    return _reactiveModelState.snapState.isWaiting;
  }

  @override
  bool get hasData {
    OnReactiveState.addToObs?.call(this);
    return _reactiveModelState.snapState.hasData;
  }

  @override
  bool get isDone {
    OnReactiveState.addToObs?.call(this);
    return _reactiveModelState.snapState.isDone;
  }

  @override
  bool get hasError {
    OnReactiveState.addToObs?.call(this);
    return _reactiveModelState.snapState.hasError;
  }

  @override
  dynamic get error {
    OnReactiveState.addToObs?.call(this);
    return _reactiveModelState.snapState.error;
  }

  SnapState<T>? oldSnap;

  final bool _isAsyncInjected;

  int undoStackLength = 0;

  DependsOn<T>? dependsOn;
  Timer? _dependentDebounceTimer;

  ///Create the state
  dynamic middleCreator(
    dynamic Function() crt,
    dynamic Function()? creatorMock,
  ) {
    final creator = creatorMock ?? crt;
    if (undoRedoPersistState?.persistanceProvider != null) {
      final val = snapState._infoMessage == kInitMessage
          ? undoRedoPersistState!.persistedCreator()
          : null;

      dynamic recreate(T? val) {
        final shouldRecreate =
            undoRedoPersistState!.persistanceProvider!.shouldRecreateTheState;
        if (shouldRecreate == true ||
            (shouldRecreate == null && creator is Stream Function())) {
          if (val != null) {
            _reactiveModelState._initialState = val;
            //See issue 192
            final old = onInitialized;
            onInitialized = (_) {
              //Force hasData flag to be true.
              _reactiveModelState._snapState =
                  _reactiveModelState._snapState._copyToHasData(val);
              old?.call(_);
            };
          }
          return creator();
        }

        return val ?? creator();
      }

      if (val is Future) {
        return () async {
          final r = await val;
          return recreate(r);
        }();
      }
      return recreate(val);
    }

    return creator();
  }

  ///Initialize the state
  void initialize() {
    if (_reactiveModelState._isInitialized) {
      return;
    }
    final cachedAddToObs = OnReactiveState.addToObs;
    OnReactiveState.addToObs = null;
    _reactiveModelState._isInitialized = true;
    final creatorMock = cachedCreatorMocks.last;
    bool isInitializing = true;

    _reactiveModelState._setInitialStateCreator(
      middleCreator: (crt) {
        isInitializing = true;
        return middleCreator(crt, creatorMock);
      },
      middleState: (snap) {
        if (isInitializing) {
          isInitializing = false;
          if (snap.data is ChangeNotifier) {
            (snap.data as ChangeNotifier).addListener(notify);
            _reactiveModelState.listeners.addCleaner(() {
              (snap.data as ChangeNotifier).removeListener(notify);
            });
            return snap;
          }
          if (snapState._infoMessage == kRecomputing) {
            return middleSnap(snap);
          }

          if (snapState._infoMessage == kRefreshMessage) {
            return middleSnap(
              snap.hasData
                  ? snap._copyToIsIdle()
                  : snap.copyWith(data: _reactiveModelState._initialState),
            );
          } else if (snapState._infoMessage != kInitMessage) {
            return middleSnap(snap);
          }
          if (snap.hasData) {
            snap =
                _isAsyncInjected ? snap : snap._copyToIsIdle(isActive: false);
          }
          _reactiveModelState._snapState = middleSnap(snap) ?? snap;
          return null;
        }
        return middleSnap(snap);
      },
      onDone: (snap) {
        return snap;
      },
    );

    if (dependsOn != null) {
      _setCombinedSnap(
        dependsOn!.injected,
        isInitializing: true,
      );

      _subscribeForCombinedSnap(dependsOn!.injected);
      // if (hasObservers) {
      // } else {
      //   _reactiveModelState.listeners.onFirstListerAdded = () {
      //     _subscribeForCombinedSnap(dependsOn!.injected);
      //   };
      // }
    } else {
      _reactiveModelState._initialStateCreator!();
    }
    if (_reactiveModelState._isDisposed) {
      _reactiveModelState._isDisposed = false;
      onInitialized?.call(_reactiveModelState._snapState.data);
    }
    OnReactiveState.addToObs = cachedAddToObs;
  }

  @override
  Future<T?> refresh() async {
    if (inheritedInjects.isNotEmpty) {
      snapState = snapState._copyWith(infoMessage: kRefreshMessage);
      for (final inj in inheritedInjects) {
        inj._reactiveModelState._refresh(
          infoMessage: kRecomputing,
        );
      }
      snapState = snapState._copyWith(infoMessage: '');
      return snapState.data;
    }

    return super.refresh();
  }

  @override
  toggle() {
    super.toggle();
    undoRedoPersistState?.call(snapState, this);
  }

  @override
  SnapState<T>? _middleSnap(
    SnapState<T> s, {
    On<void>? onSetState,
    SnapState<T>? Function(MiddleSnapState<T>)? middleSnapState,
    bool shouldOverrideGlobalSideEffects = false,
    void Function(T data)? onData,
    void Function(dynamic error)? onError,
  }) =>
      middleSnap(
        s,
        onSetState: onSetState,
        shouldOverrideGlobalSideEffects: shouldOverrideGlobalSideEffects,
        onData: onData,
        onError: onError,
        middleSnapState: middleSnapState,
      );
  SnapState<T>? middleSnap(
    SnapState<T> snap, {
    On<void>? onSetState,
    SnapState<T>? Function(MiddleSnapState<T>)? middleSnapState,
    bool shouldOverrideGlobalSideEffects = false,
    void Function(T data)? onData,
    void Function(dynamic error)? onError,
  }) {
    if (!_reactiveModelState._isDisposed) {
      snap = middleSnapState?.call(
            MiddleSnapState(snapState, snap),
          ) ??
          snap;
      snap = this.middleSnapState?.call(
                MiddleSnapState(snapState, snap),
              ) ??
          snap;
    }
    if (snap is SkipSnapState) {
      return null;
    }
    oldSnap = snapState;
    if (snap.isWaiting) {
      if (snapState.isWaiting) {
        return null;
      }
      _reactiveModelState._snapState = snap;
      if (onSetState != null) {
        onSetState.call(snap);
      }
      if (!shouldOverrideGlobalSideEffects) {
        this.onSetState?.call(snap);
      }
      onWaiting?.call();
    } else if (snap.hasError) {
      if (snap.error == snapState.error) {
        return null;
      }
      _reactiveModelState._snapState = snap;
      if (onSetState != null) {
        onSetState.call(snap);
      }
      if (!shouldOverrideGlobalSideEffects) {
        this.onSetState?.call(snap);
      }

      if (onError != null) {
        onError.call(snap.error);
      } else {
        this.onError?.call(snap.error, snap.stackTrace);
      }
    } else if (snap.hasData) {
      if (snap._isImmutable == true && snap == snapState) {
        return null;
      }
      _reactiveModelState._snapState = snap;
      if (onSetState != null) {
        onSetState.call(snap);
      }
      if (!shouldOverrideGlobalSideEffects) {
        this.onSetState?.call(snap);
      }

      if (onData != null) {
        onData.call(snap.data as T);
      } else {
        onDataForSideEffect?.call(snap.data as T);
      }
      undoRedoPersistState?.call(snap, this);
    } else if (snap.isIdle) {
      undoRedoPersistState?.call(snap, this);
    }

    assert(() {
      if (toDebugString != null) {
        MiddleSnapState(oldSnap!, snap).print(
          stateToString: toDebugString,
          preMessage: debugPrintWhenNotifiedPreMessage!,
        );
      } else if (debugPrintWhenNotifiedPreMessage != null) {
        MiddleSnapState(oldSnap!, snap)
            .print(preMessage: debugPrintWhenNotifiedPreMessage!);
      }

      return true;
    }());
    if (_reactiveModelState._removeFromInjectedList == null && !snap.isIdle) {
      _reactiveModelState._removeFromInjectedList = addToInjectedModels(this);
    }

    return snap;
  }

  final _dependentDisposers = <VoidCallback>[];
  @override
  void notify() {
    super.notify();
    onSetState?.call(snapState);
  }

  void _subscribeForCombinedSnap(Set<ReactiveModel> depends) {
    for (var depend in dependsOn!.injected) {
      void fn(_) {
        if (dependsOn!.shouldNotify?.call(_nullableState) == false) {
          return;
        }

        if (dependsOn!.debounceDelay > 0) {
          subscription?.cancel();
          _dependentDebounceTimer?.cancel();
          _dependentDebounceTimer = Timer(
            Duration(milliseconds: dependsOn!.debounceDelay),
            () {
              _setCombinedSnap(dependsOn!.injected);
              // _reactiveModelState._refresh(infoMessage: kRecomputing);
              _dependentDebounceTimer = null;
            },
          );
          return;
        } else if (dependsOn!.throttleDelay > 0) {
          if (_dependentDebounceTimer != null) {
            return;
          }
          _dependentDebounceTimer = Timer(
            Duration(milliseconds: dependsOn!.throttleDelay),
            () {
              _dependentDebounceTimer = null;
            },
          );
        }

        _setCombinedSnap(dependsOn!.injected);
        // _reactiveModelState._refresh(infoMessage: kRecomputing);
      }

      void addListenerForRebuild() {
        final disposer =
            depend._reactiveModelState.listeners.addListenerForRebuild(
          fn,
          clean: () {
            if (depend.autoDisposeWhenNotUsed && !depend.hasObservers) {
              depend.dispose();
            }
          },
        );
        _dependentDisposers.add(disposer);
      }

      // if (!hasObservers) {

      final disposer =
          depend._reactiveModelState.listeners.addListenerForSideEffect(
        fn,
        clean: () {
          if (depend.autoDisposeWhenNotUsed && !depend.hasObservers) {
            depend.dispose();
          }
        },
      );
      _reactiveModelState.listeners.onFirstListerAdded = () {
        _reactiveModelState.listeners._sideEffectListeners.remove(fn);
        _dependentDisposers.remove(disposer);
        addListenerForRebuild();
      };
      depend._reactiveModelState.listeners.addCleaner(() {
        if (!hasObservers && _imp.autoDisposeWhenNotUsed) {
          dispose();
        }
      });
      _dependentDisposers.add(disposer);
      // } else {
      //   addListenerForRebuild();
      // }
    }
  }

  void _setCombinedSnap(
    Set<ReactiveModel> depends, {
    bool isInitializing = false,
  }) {
    bool isIdle = false;
    // bool isWaiting = false;

    dynamic error;
    late StackTrace stackTrace;
    late VoidCallback refresher;

    for (var depend in depends) {
      //Use _reactiveModelState.snapState.isWaiting instead of depend.isWaiting
      //to avoid dependent state to subscribe to OnReactive
      if (depend._reactiveModelState.snapState.isWaiting) {
        final snap = _reactiveModelState._snapState._copyToIsWaiting(
          data: depend._reactiveModelState._oldSnapState._infoMessage ==
                  kRefreshMessage
              ? _reactiveModelState._initialState
              : null,
          infoMessage: kDependsOn,
        );
        if (isInitializing) {
          _reactiveModelState
            .._snapState = middleSnap(snap) ?? snap
            ..setSnapStateAndRebuild = null;
        } else {
          _reactiveModelState.setSnapStateAndRebuild = middleSnap(snap);
        }
        return;
      }
      if (depend._reactiveModelState.snapState.hasError) {
        error = depend._reactiveModelState.snapState.error;
        stackTrace = depend._reactiveModelState._snapState.stackTrace!;
        refresher = depend._reactiveModelState._snapState.onErrorRefresher!;
      }

      if (depend._reactiveModelState.snapState.isIdle) {
        isIdle = true;
      }
    }

    if (error != null) {
      final snap = _reactiveModelState._snapState._copyToHasError(
        error,
        refresher,
        stackTrace: stackTrace,
      );
      if (isInitializing) {
        _reactiveModelState
          .._snapState = middleSnap(snap) ?? snap
          ..setSnapStateAndRebuild = null;
      } else {
        _reactiveModelState.setSnapStateAndRebuild = middleSnap(snap);
      }

      return;
    }

    if (isInitializing) {
      _reactiveModelState._refresh(infoMessage: kInitMessage);
    } else {
      final creator = cachedCreatorMocks.last ?? _reactiveModelState.creator;
      _reactiveModelState.setStateFn(
        (state) => creator(),
        middleState: _middleSnap,
        onDone: (_) => _,
        isWaitingForAsyncTask: false,
      )();
    }
  }

  @override
  Future<F> Function() future<F>(Future<F> Function(T s) future) {
    return () async {
      late F data;
      await future(_state).then((d) {
        if (d is T) {
          snapState = snapState.copyToHasData(d);
          onSetState?.call(snapState);
          onDataForSideEffect?.call(_state);
        }
        data = d;
      }).catchError((e, StackTrace s) {
        snapState = snapState._copyToHasError(
          e,
          () => this.future(future),
          stackTrace: s,
        );
        onSetState?.call(snapState);
        onError?.call(e, s);
        throw e;
      });
      return data;
    };
  }

  @override
  void dispose() {
    if (!_reactiveModelState._isDisposed) {
      onDisposed?.call(_state);
      if (cachedCreatorMocks.length > 1) {
        cachedCreatorMocks.removeLast();
      }
      undoRedoPersistState?.clearUndoStack();
      if (null is T || snapState.data != null) {
        undoRedoPersistState?.persistOnDispose(_state);
      }
      final middleSnap = MiddleSnapState(
        snapState,
        SnapState._nothing(
          snapState.data,
          kDisposeMessage,
          debugPrintWhenNotifiedPreMessage,
        ),
      );
      // middleSnapState?.call(middleSnap);

      assert(
        () {
          if (toDebugString != null) {
            middleSnap.print(
              stateToString: toDebugString,
              preMessage: debugPrintWhenNotifiedPreMessage!,
            );
          } else if (debugPrintWhenNotifiedPreMessage != null) {
            middleSnap.print(preMessage: debugPrintWhenNotifiedPreMessage!);
          }
          return true;
        }(),
      );
      if (dependsOn != null) {
        for (final disposer in _dependentDisposers) {
          disposer();
        }
      }
      _reactiveModelState._removeFromInjectedList?.call();
      _reactiveModelState._removeFromInjectedList = null;

      super.dispose();
    }
  }

  List<dynamic Function()?> cachedCreatorMocks = [null];

  void _injectMock(dynamic Function() fakeCreator) {
    dispose();
    RM.disposeAll();
    cachedCreatorMocks.add(fakeCreator);
  }

  @override
  void injectMock(T Function() fakeCreator) {
    _injectMock(fakeCreator);
  }

  @override
  void injectFutureMock(Future<T> Function() fakeCreator) {
    _injectMock(fakeCreator);
  }

  @override
  void injectStreamMock(Stream<T> Function() fakeCreator) {
    _injectMock(fakeCreator);
  }

  @override
  void undoState() {
    final snap = undoRedoPersistState?.undoState();
    if (snap == null) {
      return;
    }
    assert(() {
      if (toDebugString != null) {
        MiddleSnapState(snapState, snap).print(
          stateToString: toDebugString,
          preMessage: debugPrintWhenNotifiedPreMessage! + " (UndoState)",
        );
      } else if (debugPrintWhenNotifiedPreMessage != null) {
        MiddleSnapState(snapState, snap).print(
            preMessage: debugPrintWhenNotifiedPreMessage! + " (UndoState)");
      }

      return true;
    }());
    // if (snap != snapState) {}
    _reactiveModelState.setSnapStateAndRebuild = snap;
  }

  @override
  void redoState() {
    final snap = undoRedoPersistState?.redoState();
    if (snap == null) {
      return;
    }
    assert(() {
      if (toDebugString != null) {
        MiddleSnapState(snapState, snap).print(
          stateToString: toDebugString,
          preMessage: debugPrintWhenNotifiedPreMessage! + " (RedoState)",
        );
      } else if (debugPrintWhenNotifiedPreMessage != null) {
        MiddleSnapState(snapState, snap).print(
            preMessage: debugPrintWhenNotifiedPreMessage! + " (RedoState)");
      }

      return true;
    }());
    _reactiveModelState.setSnapStateAndRebuild = snap;
  }

  @override
  void clearUndoStack() {
    undoRedoPersistState?.clearUndoStack();
    notify();
  }

  @override
  bool get canRedoState {
    OnReactiveState.addToObs?.call(this);
    return undoRedoPersistState?.canRedoState ?? false;
  }

  @override
  bool get canUndoState {
    OnReactiveState.addToObs?.call(this);
    return undoRedoPersistState?.canUndoState ?? false;
  }

  @override
  void persistState() {
    undoRedoPersistState?.persistanceProvider?.write(_state);
  }

  @override
  void deletePersistState() {
    undoRedoPersistState?.deleteState(this);
  }

  @override
  String toString() {
    return 'Injected#$hashCode($snapState)';
  }

  final inheritedInjects = <Injected<T>>{};
  bool? _shouldContextWithGlobal;
  @override
  Widget inherited({
    required Widget Function(BuildContext) builder,
    Key? key,
    required FutureOr<T> Function()? stateOverride,
    bool? connectWithGlobal,
    SideEffects? sideEffects,
    String? debugPrintWhenNotifiedPreMessage,
    String Function(T?)? toDebugString,
  }) {
    if (connectWithGlobal == null && _shouldContextWithGlobal == null) {
      if (!_reactiveModelState._isInitialized) {
        try {
          final s = _reactiveModelState.creator();
          _reactiveModelState
            .._isInitialized = true
            .._isDisposed = false;
          if (s is T) {
            snapState = snapState._copyWith(data: s);
          }
          _reactiveModelState.setStateFn(
            (_) => s,
            middleState: _middleSnap,
            onDone: (_) {},
          )();
        } catch (e) {
          if (e is! UnimplementedError) {
            rethrow;
          }
        }
      }

      _shouldContextWithGlobal = _reactiveModelState._snapState.data == null;
    }

    return _inherited(
      builder: builder,
      key: key,
      stateOverride: stateOverride,
      connectWithGlobal: connectWithGlobal ?? _shouldContextWithGlobal!,
      sideEffects: sideEffects,
      debugPrintWhenNotifiedPreMessage: debugPrintWhenNotifiedPreMessage,
      toDebugString: toDebugString,
    );
  }

  @override
  Widget reInherited({
    Key? key,
    required BuildContext context,
    required Widget Function(BuildContext) builder,
  }) {
    final globalInjected = (context
            .getElementForInheritedWidgetOfExactType<_InheritedInjected<T>>()
            ?.widget as _InheritedInjected<T>?)
        ?.globalInjected;
    assert(() {
      if (globalInjected == null) {
        StatesRebuilerLogger.log(
          'Can not find and Inherited state ancestor',
          'The provided BuildContext has no Inherited state ancestor',
        );
        return false;
      }

      return true;
    }());

    return _inherited(
      key: key ?? Key('$context'),
      builder: builder,
      globalInjected: globalInjected,
      reInheritedInject: globalInjected!(context),
      debugPrintWhenNotifiedPreMessage: debugPrintWhenNotifiedPreMessage,
      toDebugString: toDebugString,
    );
  }

  Widget _inherited({
    required Widget Function(BuildContext) builder,
    Key? key,
    FutureOr<T> Function()? stateOverride,
    Injected<T>? reInheritedInject,
    Injected<T>? globalInjected,
    bool connectWithGlobal = true,
    SideEffects? sideEffects,
    String? debugPrintWhenNotifiedPreMessage,
    Object? Function(T?)? toDebugString,
  }) {
    return StateBuilderBase(
      (widget, setState) {
        //Not always globalInjected == this
        globalInjected ??= this;
        late Injected<T> injected;
        late VoidCallback disposer;

        if (stateOverride == null) {
          injected = reInheritedInject ?? this;
        } else {
          injected = InjectedImp<T>(
            creator: () {
              try {
                return stateOverride();
              } catch (e) {
                if (e is RangeError) {
                  return injected.snapState.data;
                }
                rethrow;
              }
            },
            onSetState: sideEffects != null
                ? On(() => sideEffects.onSetState?.call(injected.snapState))
                : null,
            // initialState: state,
            onInitialized: (_) {
              if (connectWithGlobal) {
                _reactiveModelState
                  .._isInitialized = true
                  .._isDisposed = false;
                inheritedInjects.add(injected);
                _setCombinedInheritedSnap(inheritedInjects, injected);
              }
            },
            debugPrintWhenNotifiedPreMessage: debugPrintWhenNotifiedPreMessage,
            toDebugString: toDebugString,
          );
        }
        return LifeCycleHooks<_InheritedHelper>(
          mountedState: (context) {
            disposer =
                injected._reactiveModelState.listeners.addListenerForRebuild(
              (snap) {
                if (snapState._infoMessage == kRefreshMessage) {
                  setState();
                  return;
                }
                if (inheritedInjects.isNotEmpty) {
                  _setCombinedInheritedSnap(inheritedInjects, injected);
                }
                setState();
              },
              clean: () {
                if (injected != this) {
                  if (autoDisposeWhenNotUsed && !hasObservers) {
                    dispose();
                  }
                  inheritedInjects.remove(injected);
                }
                if (injected.autoDisposeWhenNotUsed) {
                  injected.dispose();
                }
              },
            );
          },
          dispose: (context) {
            // if (injected != this) {
            //   print(inheritedInjects);
            //   inheritedInjects.remove(injected);
            //   print(inheritedInjects);
            // }
            disposer();
          },
          builder: (context, widget) {
            return _InheritedInjected(
              injected: injected,
              globalInjected: globalInjected!,
              context: context,
              child: Builder(builder: (context) => widget.builder(context)),
            );
          },
        );
      },
      key: key,
      widget: _InheritedHelper(
        builder: builder,
      ),
    );
  }

  void _setCombinedInheritedSnap(
      Set<Injected<T>> inheritStates, Injected<T> inj) {
    bool isWaiting = false;

    dynamic error;
    late StackTrace stackTrace;
    late VoidCallback refresher;
    SnapState<T>? oldSnap;
    SnapState<T>? newSnap;
    final cachedAddToObs = OnReactiveState.addToObs;
    OnReactiveState.addToObs = null;
    for (var state in inheritStates) {
      if (state.isWaiting) {
        isWaiting = true;
        oldSnap = state._imp.oldSnap;
        break;
      }
      if (state.hasError) {
        error = state.error;
        stackTrace = state._reactiveModelState._snapState.stackTrace!;
        refresher = state._reactiveModelState._snapState.onErrorRefresher!;
        oldSnap = state._imp.oldSnap;
      }
    }

    if (isWaiting) {
      newSnap = _reactiveModelState._snapState
          ._copyToIsWaiting(infoMessage: kInherited);
    }
    if (error != null) {
      newSnap = _reactiveModelState._snapState.copyToHasError(
        error,
        stackTrace: stackTrace,
        onErrorRefresher: refresher,
      );
    }

    snapState = oldSnap?._copyWith(
          debugPrintWhenNotifiedPreMessage:
              snapState._debugPrintWhenNotifiedPreMessage,
        ) ??
        inj._imp.oldSnap!._copyWith(
          debugPrintWhenNotifiedPreMessage:
              snapState._debugPrintWhenNotifiedPreMessage,
        );
    _reactiveModelState.setSnapStateAndRebuild = middleSnap(
      newSnap?._copyWith(
            debugPrintWhenNotifiedPreMessage:
                snapState._debugPrintWhenNotifiedPreMessage,
          ) ??
          inj.snapState._copyWith(
            debugPrintWhenNotifiedPreMessage:
                snapState._debugPrintWhenNotifiedPreMessage,
          ),
    );
    OnReactiveState.addToObs = cachedAddToObs;
  }
}

class _InheritedInjected<T> extends InheritedWidget {
  _InheritedInjected({
    Key? key,
    required Widget child,
    required this.injected,
    required this.globalInjected,
    required this.context,
  }) : super(key: key, child: child) {
    final data = injected._reactiveModelState.snapState.data;
    state = data;
  }
  final Injected<T> injected;
  late final T? state;
  final Injected<T> globalInjected;
  final BuildContext context;

  @override
  bool updateShouldNotify(_InheritedInjected _) {
    return _.state != state;
  }
}

class _InheritedHelper {
  final Widget Function(BuildContext) builder;
  _InheritedHelper({
    required this.builder,
  });
}
